<?php

namespace Drupal\heading_highlighter\Plugin\Block;

use Drupal\Component\Utility\Html;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a Heading Highlighter block.
 *
 * @Block(
 *   id = "heading_highlighter",
 *   admin_label = @Translation("Heading Highlighter"),
 *   category = @Translation("Heading Highlighter")
 * )
 */
class HeadingHighlighterBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * Configuration Factory.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * HeadingHighlighterBlock constructor.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param Drupal\Core\Config\ConfigFactory $configFactory
   *   The configuration manager.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ConfigFactory $configFactory) {
    $this->configuration = $configuration;
    $this->pluginId = $plugin_id;
    $this->pluginDefinition = $plugin_definition;
    $this->configFactory = $configFactory;
  }

  /**
   * Creates an instance of the HeadingHighlighterBlock plugin.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The container to pull out services used in the plugin.
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   *
   * @return static
   *   Returns an instance of this plugin.
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $configFactory = $container->get('config.factory');
    return new static($configuration, $plugin_id, $plugin_definition, $configFactory);
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'show_toggle' => 1,
      'highlight_default' => 1,
      'default_styling' => 1,
      'container' => 'none',
      'container_region' => '',
      'container_selector' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $config = $this->configuration;

    $form['show_toggle'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show toggle'),
      '#default_value' => $config['show_toggle'],
      '#description' => $this->t('Whether or not to show the heading highlighter toggle button on the page.'),
    ];

    $form['highlight_default'] = [
      '#type' => 'radios',
      '#title' => $this->t('Highlight default'),
      '#default_value' => $config['highlight_default'],
      '#options' => [
        0 => $this->t('Disable highlight until toggled on'),
        1 => $this->t('Highlight until toggled off'),
      ],
      '#states' => [
        'visible' => [
          ':input[name="settings[show_toggle]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['default_styling'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use default CSS styles'),
      '#default_value' => $config['default_styling'],
      '#description' => $this->t('Use the default CSS styles that comes with the module.'),
    ];

    $form['container'] = [
      '#type' => 'radios',
      '#title' => $this->t('Contain highlighter to part of the page'),
      '#default_value' => $config['container'],
      '#options' => [
        'none' => $this->t('No container, highlight all page headings'),
        'region' => $this->t('Specific Region'),
        'selector' => $this->t('Within CSS selector'),
      ],
    ];

    $container_input = ':input[name="settings[container]"]';

    $theme = $this->configFactory->get('system.theme')->get('default');
    $regions = system_region_list($theme);

    $form['container_region'] = [
      '#type' => 'select',
      '#title' => 'Select container region',
      '#default_value' => $config['container_region'],
      '#options' => $regions,
      '#states' => [
        'visible' => [
          $container_input => ['value' => 'region'],
        ],
        'required' => [
          $container_input => ['value' => 'region'],
        ],
      ],
    ];

    $form['container_selector'] = [
      '#type' => 'textfield',
      '#title' => 'Enter CSS container selector',
      '#default_value' => $config['container_selector'],
      '#size' => 60,
      '#maxlength' => 128,
      '#states' => [
        'visible' => [
          $container_input => ['value' => 'selector'],
        ],
        'required' => [
          $container_input => ['value' => 'selector'],
        ],
      ],
    ];

    return $form;

  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    parent::blockSubmit($form, $form_state);

    $this->configuration['show_toggle'] = $form_state
      ->getValue('show_toggle');
    $this->configuration['highlight_default'] = $form_state
      ->getValue('highlight_default');
    $this->configuration['default_styling'] = $form_state
      ->getValue('default_styling');
    $this->configuration['container'] = $form_state
      ->getValue('container');
    $this->configuration['container_region'] = $form_state
      ->getValue('container_region');
    $this->configuration['container_selector'] = $form_state
      ->getValue('container_selector');
  }

  /**
   * {@inheritdoc}
   */
  protected function blockAccess(AccountInterface $account) {
    return AccessResult::allowedIfHasPermission($account, 'view heading_highlighter');
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $config = $this->configuration;

    $id = Html::getUniqueId('heading-highlighter');

    $container_selector = 'body';
    if ($config['container'] === 'selector') {
      $container_selector = $config['container_selector'];
    }
    elseif ($config['container'] === 'region') {
      $container_selector = '.region.region-' . $config['container_region'];
    }

    // Only allow highlight off if the toggle is visible.
    $highlight_on = !($config['show_toggle'] && !$config['highlight_default']);

    $build = [
      'content' => [
        '#theme' => 'heading_highlighter',
        '#show_toggle' => $config['show_toggle'],
        '#id' => $id,
        '#highlight_default' => $highlight_on,
        '#attached' => [
          'library' => ['heading_highlighter/functionality'],
        ],
      ],
    ];

    $build['content']['#attached']['drupalSettings']['headingHighlighter'][] = [
      'selector' => $container_selector,
      'id' => $id,
      'show_toggle' => $config['show_toggle'],
      'highlight_default' => $highlight_on,
    ];

    if ($config['default_styling']) {
      $build['content']['#attached']['library'][] = 'heading_highlighter/styling';
    }

    return $build;
  }

}
